@@ -261,12 +261,7 @@ zip_info_unref (ZipInfo *info)
 	g_free (info);
 }
 
-/**
- * zip_dup :
- * @src :
- *
- * Return value: the partial duplicate.
- **/
+/* Returns a partial duplicate. */
 static GsfInfileZip *
 zip_dup (GsfInfileZip const *src, GError **err)
 {
@@ -277,6 +272,7 @@ zip_dup (GsfInfileZip const *src, GError **err)
 	dst = g_object_new (GSF_INFILE_ZIP_TYPE,
 			    "internal-parent", src,
 			    NULL);
+	if (G_UNLIKELY (NULL == dst)) return NULL;
 
 	if (dst->err) {
 		if (err)
@@ -290,12 +286,12 @@ zip_dup (GsfInfileZip const *src, GError **err)
 
 /**
  * zip_read_dirents:
- * @zip :
+ * @zip : #GsfInfileZip
  *
  * Read zip headers and do some sanity checking
  * along the way.
  *
- * Return value: TRUE on error setting zip->err.
+ * Returns: %TRUE on error setting zip->err.
  **/
 static gboolean
 zip_read_dirents (GsfInfileZip *zip)
@@ -364,12 +360,12 @@ zip_build_vdirs (GsfInfileZip *zip)
 
 /**
  * zip_init_info :
- * @zip :
+ * @zip : #GsfInfileZip
  *
  * Read zip headers and do some sanity checking
  * along the way.
  *
- * Return value: TRUE on error setting zip->err.
+ * Returns: TRUE on error setting zip->err.
  **/
 static gboolean
 zip_init_info (GsfInfileZip *zip)
@@ -415,9 +411,8 @@ zip_child_init (GsfInfileZip *child, GError **errmsg)
 	}
 
 	if (NULL != err) {
-		g_print (err);
 		if (errmsg != NULL)
-			*errmsg = g_error_new (gsf_input_error_id (), 0, err);
+			*errmsg = g_error_new_literal (gsf_input_error_id (), 0, err);
 		g_free (err);
 		return TRUE;
 	}
@@ -848,9 +843,9 @@ gsf_infile_zip_class_init (GObjectClass *gobject_class)
 				      G_PARAM_CONSTRUCT_ONLY));
 }
 
-GSF_DYNAMIC_CLASS (GsfInfileZip, gsf_infile_zip,
-		   gsf_infile_zip_class_init, gsf_infile_zip_init,
-		   GSF_INFILE_TYPE)
+GSF_CLASS (GsfInfileZip, gsf_infile_zip,
+	   gsf_infile_zip_class_init, gsf_infile_zip_init,
+	   GSF_INFILE_TYPE)
 
 /**
  * gsf_infile_zip_new :
@@ -872,6 +867,8 @@ gsf_infile_zip_new (GsfInput *source, GError **err)
 	zip = g_object_new (GSF_INFILE_ZIP_TYPE,
 			    "source", source,
 			    NULL);
+	if (G_UNLIKELY (NULL == zip)) return NULL;
+
 	if (zip->err) {
 		if (err)
 			*err = g_error_copy (zip->err);
