@@ -45,12 +45,12 @@ glade_enum_from_string (GType type, const char *string)
     if (endptr != string) /* parsed a number */
 	return ret;
 
-    eclass = g_type_class_ref(type);
-    ev = g_enum_get_value_by_name(eclass, string);
-    if (!ev) ev = g_enum_get_value_by_nick(eclass, string);
+    eclass = g_type_class_ref (type);
+    ev = g_enum_get_value_by_name (eclass, string);
+    if (!ev) ev = g_enum_get_value_by_nick (eclass, string);
     if (ev)  ret = ev->value;
 
-    g_type_class_unref(eclass);
+    g_type_class_unref (eclass);
 
     return ret;
 }
@@ -61,11 +61,11 @@ glade_string_from_enum (GType type, gint value)
     GEnumClass *eclass;
     GEnumValue *ev;
 
-    eclass = g_type_class_ref(type);
+    eclass = g_type_class_ref (type);
 
     ev = g_enum_get_value (eclass, value);
 
-    g_type_class_unref(eclass);
+    g_type_class_unref (eclass);
 
     return ev ? ev->value_name : "0";
 }
@@ -350,12 +350,12 @@ gsf_xml_parser_context (GsfInput *input)
 
 /**
  * gsf_xml_output_buffer_new :
- * @output:
+ * @output: #GsfOutput
  * @encoding: optionally %NULL.
  *
  * <note>This adds a reference to @output.</note>
  * <note>This is <emphasis>not</emphasis> releated to #GsfXMLOut.</note>
- */
+ **/
 static xmlOutputBufferPtr
 gsf_xml_output_buffer_new (GsfOutput *output,
 			   xmlCharEncodingHandlerPtr handler)
@@ -371,6 +371,17 @@ gsf_xml_output_buffer_new (GsfOutput *output,
 	return res;
 }
 
+/**
+ * gsf_xmlDocFormatDump :
+ * @output : #GsfOutput
+ * @cur : #xmlDocPtr
+ * @encoding : The encoding to use.
+ * @format : %TRUE to reformat the output.
+ *
+ * Dumps the document @cur into @output.
+ *
+ * Returns: status from xmlSaveFormatFileTo.
+ **/
 int
 gsf_xmlDocFormatDump (GsfOutput *output, xmlDocPtr cur, char const *encoding,
 		      gboolean format)
@@ -408,6 +419,101 @@ gsf_xmlDocFormatDump (GsfOutput *output, xmlDocPtr cur, char const *encoding,
 	return xmlSaveFormatFileTo (buf, cur, encoding, format);
 }
 
+/**************************************************************************/
+/* We parse and do some limited validation of the XML file, if this passes,
+ * then we return TRUE
+ */
+typedef struct {
+	GsfXMLProbeFunc	func;
+	gboolean success;
+} GsfXMLProbeState;
+
+static void
+gsf_xml_probe_error (GsfXMLProbeState *state, char const *msg, ...)
+{
+	state->func = NULL;
+	state->success = FALSE;
+}
+static void
+gsf_xml_probe_element (GsfXMLProbeState *state,
+		       const xmlChar *name,
+		       const xmlChar *prefix,
+		       const xmlChar *URI,
+		       int nb_namespaces,
+		       const xmlChar **namespaces,
+		       int nb_attributes,
+		       int nb_defaulted,
+		       const xmlChar **attributes)
+{
+	state->success = (state->func) (name, prefix, URI, nb_namespaces, namespaces,
+					nb_attributes, nb_defaulted, attributes);
+	state->func = NULL;
+}
+
+gboolean
+gsf_xml_probe (GsfInput *input, GsfXMLProbeFunc func)
+{
+	static xmlSAXHandler gsf_xml_prober = {
+		NULL, /* internalSubset */
+		NULL, /* isStandalone */
+		NULL, /* hasInternalSubset */
+		NULL, /* hasExternalSubset */
+		NULL, /* resolveEntity */
+		NULL, /* getEntity */
+		NULL, /* entityDecl */
+		NULL, /* notationDecl */
+		NULL, /* attributeDecl */
+		NULL, /* elementDecl */
+		NULL, /* unparsedEntityDecl */
+		NULL, /* setDocumentLocator */
+		NULL, /* startDocument */
+		NULL, /* endDocument */
+		NULL, /* startElement */
+		NULL, /* endElement */
+		NULL, /* reference */
+		NULL, /* characters */
+		NULL, /* ignorableWhitespace */
+		NULL, /* processingInstruction */
+		NULL, /* comment */
+		NULL, /* xmlParserWarning */
+		(errorSAXFunc) &gsf_xml_probe_error, /* error */
+		(errorSAXFunc) &gsf_xml_probe_error, /* fatalError */
+		NULL, /* getParameterEntity */
+		NULL, /* cdataBlock; */
+		NULL, /* externalSubset; */
+		XML_SAX2_MAGIC,
+		NULL,
+		(startElementNsSAX2Func) &gsf_xml_probe_element, /* startElementNs */
+		NULL, /* endElementNs */
+		NULL  /* xmlStructuredErrorFunc */
+	};
+	GsfXMLProbeState probe_state = { func, FALSE };
+	xmlParserCtxt *parse_context;
+	char const *buf;
+
+	if (gsf_input_seek (input, 0, G_SEEK_SET))
+		return FALSE;
+
+	g_object_ref (input);
+	input = gsf_input_uncompress (input);
+	gsf_input_seek (input, 0, G_SEEK_SET);
+
+	buf = gsf_input_read (input, 4, NULL);
+	if (NULL != buf ) {
+		parse_context = xmlCreatePushParserCtxt (&gsf_xml_prober, &probe_state,
+			(char *)buf, 4, gsf_input_name (input));
+		if (NULL != parse_context) {
+			while (NULL != probe_state.func &&
+			       NULL != (buf = gsf_input_read (input, 1, NULL)))
+				xmlParseChunk (parse_context, (char *)buf, 1, 0);
+		}
+		xmlFreeParserCtxt (parse_context);
+	}
+	g_object_unref (input);
+
+	return probe_state.success;
+}
+
 /***************************************************************************/
 
 typedef struct {
@@ -904,8 +1010,6 @@ GsfXMLInDoc *
 gsf_xml_in_doc_new (GsfXMLInNode const *nodes, GsfXMLInNS const *ns)
 {
 	GsfXMLInDoc  *doc;
-	GsfXMLInNode const *e_node;
-	GsfXMLInNodeInternal *tmp, *node;
 
 	g_return_val_if_fail (nodes != NULL, NULL);
 
@@ -915,6 +1019,33 @@ gsf_xml_in_doc_new (GsfXMLInNode const *nodes, GsfXMLInNS const *ns)
 		NULL, (GDestroyNotify) gsf_xml_in_node_internal_free);
 	doc->ns        = ns;
 
+	gsf_xml_in_doc_add_nodes (doc, nodes);
+
+	if (NULL == doc->root_node) {
+		gsf_xml_in_doc_free (doc);
+		g_return_val_if_fail (NULL != doc->root_node, NULL);
+	}
+
+	return doc;
+}
+
+/**
+ * gsf_xml_in_doc_add_nodes :
+ * @doc : #GsfXMLInDoc
+ * @nodes : %NULL terminated array of #GsfXMLInNode
+ *
+ * Adds additional nodes to the structure of @doc
+ **/
+void
+gsf_xml_in_doc_add_nodes (GsfXMLInDoc *doc,
+			  GsfXMLInNode const *nodes)
+{
+	GsfXMLInNode const *e_node;
+	GsfXMLInNodeInternal *tmp, *node;
+
+	g_return_if_fail (doc != NULL);
+	g_return_if_fail (nodes != NULL);
+
 	for (e_node = nodes; e_node->id != NULL ; e_node++ ) {
 		node = g_hash_table_lookup (doc->symbols, e_node->id);
 		if (node != NULL) {
@@ -941,7 +1072,7 @@ gsf_xml_in_doc_new (GsfXMLInNode const *nodes, GsfXMLInNS const *ns)
 			g_hash_table_insert (doc->symbols,
 				(gpointer)node->pub.id, node);
 		}
-		if (e_node == nodes) /* first valid node is the root */
+		if (NULL == doc->root_node && e_node == nodes) /* first valid node is the root */
 			doc->root_node = node;
 
 		/* NOTE : use e_node for parent_id rather than node
@@ -967,8 +1098,6 @@ gsf_xml_in_doc_new (GsfXMLInNode const *nodes, GsfXMLInNS const *ns)
 			continue;
 		}
 	}
-
-	return doc;
 }
 
 /**
@@ -990,7 +1119,7 @@ gsf_xml_in_doc_set_unknown_handler (GsfXMLInDoc *doc,
  * gsf_xml_in_push_state :
  * @xin : #GsfXMLIn
  * @doc : #GsfXMLInDoc
- * @new_state :
+ * @new_state : arbitrary content for the parser
  * @dtor : #GsfXMLInExtDtor
  * @attrs : array of xmlChar const *
  *
@@ -1023,7 +1152,7 @@ gsf_xml_in_push_state (GsfXMLIn *xin, GsfXMLInDoc const *doc,
  * gsf_xml_in_doc_parse :
  * @doc : #GsfXMLInDoc
  * @input : #GsfInput
- * @user_state :
+ * @user_state : arbitrary content stored in the parser
  *
  * Read an xml document from @input and parse based on the the descriptor in
  * @doc
@@ -1073,13 +1202,13 @@ gsf_xml_in_get_input (GsfXMLIn const *xin)
 /**
  * gsf_xml_in_check_ns :
  * @xin : #GsfXMLIn
- * @str :
- * @ns_id :
+ * @str : string to check
+ * @ns_id : the namespace id
  * 
  * According to @state is @str in the namespace @ns_id ?
  *
  * Returns: a pointer to @str after the namespace if successful,
- * 	otherwise NULL.
+ * 	otherwise %NULL.
  **/
 char const *
 gsf_xml_in_check_ns (GsfXMLIn const *xin, char const *str, unsigned int ns_id)
@@ -1108,6 +1237,9 @@ gsf_xml_in_check_ns (GsfXMLIn const *xin, char const *str, unsigned int ns_id)
  * @ns_id : The name space id to check
  * @name  : The target node name
  *
+ * Checks to see if @str is the same as @ns_id::@name with either an explicit
+ * namespace or the current default namespace.
+ *
  * Returns: %TRUE if @str == @ns_id:@name according to @state.
  **/
 gboolean
@@ -1641,8 +1773,9 @@ gsf_xml_out_add_color (GsfXMLOut *xout, char const *id,
 void
 gsf_xml_out_add_enum (GsfXMLOut *xout, char const *id, GType etype, gint val)
 {
-	GEnumClass *eclass = G_ENUM_CLASS (g_type_class_peek (etype));
+	GEnumClass *eclass = G_ENUM_CLASS (g_type_class_ref (etype));
 	GEnumValue *ev = g_enum_get_value (eclass, val);
+	g_type_class_unref (eclass);
 
 	if (ev)
 		gsf_xml_out_add_cstr_unchecked (xout, id, ev->value_name);
@@ -1757,6 +1890,14 @@ gsf_xml_out_add_base64 (GsfXMLOut *xout, char const *id,
 	g_free (tmp);
 }
 
+/**
+ * gsf_xml_out_get_output :
+ * @xout : #GsfXMLOut
+ *
+ * Get the #GsfInput we are parsing from.
+ *
+ * Returns: #GsfInput or %NULL.
+ **/
 GsfOutput *
 gsf_xml_out_get_output (GsfXMLOut const *xout)
 {
