@@ -24,7 +24,7 @@
 #include <gsf/gsf-output-impl.h>
 #include <gsf/gsf-impl-utils.h>
 #include <gsf/gsf-utils.h>
-#include <glib/gconvert.h>
+#include <glib.h>
 #include <string.h>
 
 static GObjectClass *parent_class;
@@ -176,26 +176,23 @@ gsf_output_csv_write_eol (GsfOutputCsv *csv)
 
 /* ------------------------------------------------------------------------- */
 
-static GType gsf_output_csv_quoting_mode_type = 0;
 GType
 gsf_output_csv_quoting_mode_get_type (void)
 {
-	return gsf_output_csv_quoting_mode_type;
-}
+	static GType gsf_output_csv_quoting_mode_type = 0;
 
-void
-gsf_output_csv_quoting_mode_register_type (GTypeModule *module)
-{
 	if (gsf_output_csv_quoting_mode_type == 0) {
-		static GEnumValue const values[] = {
+		GEnumValue const values[] = {
 		{ GSF_OUTPUT_CSV_QUOTING_MODE_NEVER,  (char *)"GSF_OUTPUT_CSV_QUOTING_MODE_NEVER",  (char *)"never" },
 		{ GSF_OUTPUT_CSV_QUOTING_MODE_AUTO,   (char *)"GSF_OUTPUT_CSV_QUOTING_MODE_AUTO",   (char *)"auto" },
 		{ GSF_OUTPUT_CSV_QUOTING_MODE_ALWAYS, (char *)"GSF_OUTPUT_CSV_QUOTING_MODE_ALWAYS", (char *)"always" },
 		{ 0, NULL, NULL }
 		};
-		gsf_output_csv_quoting_mode_type =
-			g_type_module_register_enum (module, "GsfOutputCsvQuotingMode", values);
+		gsf_output_csv_quoting_mode_type = g_enum_register_static (
+			g_intern_static_string ("GsfOutputCsvQuotingMode"), values);
 	}
+
+	return gsf_output_csv_quoting_mode_type;
 }
 
 /* ------------------------------------------------------------------------- */
@@ -255,8 +252,8 @@ gsf_output_csv_get_property (GObject     *object,
 static void
 gsf_output_csv_set_sink (GsfOutputCsv *csv, GsfOutput *sink)
 {
-	g_return_if_fail (GSF_IS_OUTPUT (sink));
-	g_object_ref (sink);
+	if (sink)
+		g_object_ref (sink);
 	if (csv->sink)
 		g_object_unref (csv->sink);
 	csv->sink = sink;
@@ -395,7 +392,7 @@ gsf_output_csv_class_init (GObjectClass *gobject_class)
 	parent_class = g_type_class_peek_parent (gobject_class);
 }
 
-GSF_DYNAMIC_CLASS (GsfOutputCsv, gsf_output_csv,
-		   gsf_output_csv_class_init, gsf_output_csv_init,
-		   GSF_OUTPUT_TYPE)
+GSF_CLASS (GsfOutputCsv, gsf_output_csv,
+	   gsf_output_csv_class_init, gsf_output_csv_init,
+	   GSF_OUTPUT_TYPE)
 
