@@ -73,18 +73,15 @@
  * mixture.
  */
 #define G_ARMFLOAT_ENDIAN 56781234
-#if defined(__arm__) && !defined(__vfp__) && (G_BYTE_ORDER == G_LITTLE_ENDIAN)
+#if defined(__arm__) && !defined(__ARM_EABI__) && (G_BYTE_ORDER == G_LITTLE_ENDIAN)
 #define G_FLOAT_BYTE_ORDER G_ARMFLOAT_ENDIAN
 #else
 #define G_FLOAT_BYTE_ORDER G_BYTE_ORDER
 #endif
 
 
+#ifndef HAVE_G_BASE64_ENCODE
 static void base64_init (void);
-
-#ifdef G_OS_WIN32
-#include <windows.h>
-G_WIN32_DLLMAIN_FOR_DLL_NAME (static, dll_name)
 #endif
 
 #ifdef _GSF_GTYPE_THREADING_FIXED
@@ -108,6 +105,19 @@ static GSF_CLASS (GsfDummyTypeModule, gsf_dummy_type_module,
 static GTypeModule *static_type_module = NULL;
 #endif
 
+#ifdef G_OS_WIN32
+#include <windows.h>
+static HMODULE gsf_dll_hmodule;
+BOOL WINAPI
+DllMain (HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved);
+BOOL WINAPI
+DllMain (HINSTANCE hinstDLL, DWORD fdwReason, G_GNUC_UNUSED LPVOID lpvReserved)
+{
+	if (fdwReason == DLL_PROCESS_ATTACH) gsf_dll_hmodule = hinstDLL;
+	return TRUE;
+}
+#endif
+
 /**
  * gsf_init :
  *
@@ -116,32 +126,55 @@ static GTypeModule *static_type_module = NULL;
 void
 gsf_init (void)
 {
+	static gboolean libgsf_initialized = FALSE;
+	if (libgsf_initialized)
+		return;
+
 #ifdef ENABLE_NLS
 #ifdef G_OS_WIN32
-#undef GNOMELOCALEDIR
-	gchar *prefix = g_win32_get_package_installation_directory (NULL, dll_name);
-	gchar *GNOMELOCALEDIR = g_build_filename (prefix, "lib/locale", NULL);
-	g_free (prefix);
+	{
+		char *pkg_dir	  = g_win32_get_package_installation_directory_of_module (gsf_dll_hmodule);
+		gchar *locale_dir = g_build_filename (pkg_dir, "lib/locale", NULL);
+		bindtextdomain (GETTEXT_PACKAGE, locale_dir);
+		g_free (locale_dir);
+		g_free (pkg_dir);
+	}
+#else
+	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 #endif
-	bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
-#ifdef G_OS_WIN32
-	g_free (GNOMELOCALEDIR);
-#endif
-	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 #endif
 
 	g_type_init ();
+#ifndef HAVE_G_BASE64_ENCODE
 	base64_init ();
+#endif
 
 #ifdef _GSF_GTYPE_THREADING_FIXED
 	if (NULL == static_type_module) {
 		static_type_module = g_object_new (gsf_dummy_type_module_get_type(), NULL);
+		g_assert (static_type_module != NULL);
 		g_type_module_use (static_type_module);
 		g_type_module_set_name (static_type_module, "libgsf-builtin");
 	}
 #else
 	gsf_init_dynamic (NULL);
 #endif
+
+	{
+		/* Little-endian representation of M_PI.  */
+		static guint8 pibytes[8] = {
+			0x18, 0x2d, 0x44, 0x54, 0xfb, 0x21, 0x09, 0x40
+		};
+
+		/*
+		 * If this fails, see
+		 *   http://bugzilla.gnome.org/show_bug.cgi?id=350973
+		 */
+		double pi = gsf_le_get_double (pibytes);
+		if (!(pi > 3.14 && pi < 3.15))
+			g_error ("Compilation trouble with endianess.");
+	}
 }
 
 /**
@@ -280,7 +313,7 @@ gsf_mem_dump (guint8 const *ptr, size_t len)
 /**
  * gsf_input_dump :
  * @input: a #GsfInput
- * @dump_as_hex: If TRUE, dump in hexidecmal format
+ * @dump_as_hex: If %TRUE, dump in hexidecmal format
  *
  * Dumps @input's contents to STDOUT, optionally in hex format.
  */
@@ -569,13 +602,13 @@ gsf_iconv_close (GIConv handle)
 /**
  * gsf_filename_to_utf8:
  * @filename: file name suitable for open(2).
- * @quoted: if TRUE, the resulting utf8 file name will be quoted
+ * @quoted: if %TRUE, the resulting utf8 file name will be quoted
  *    (unless it is invalid).
  *
  * A utility wrapper to make sure filenames are valid utf8.
  * Caller must g_free the result.
  *
- * Returns @filename using utf-8 encoding for display
+ * Returns: @filename using utf-8 encoding for display
  **/
 char *
 gsf_filename_to_utf8 (char const *filename, gboolean quoted)
@@ -592,6 +625,7 @@ gsf_filename_to_utf8 (char const *filename, gboolean quoted)
 	return result;
 }
 
+#ifndef HAVE_G_BASE64_ENCODE
 /***************************************************************************/
 /* some code taken from evolution/camel/camel-mime-utils.c */
 
@@ -602,7 +636,7 @@ gsf_filename_to_utf8 (char const *filename, gboolean quoted)
  *           Jeffrey Stedfast <fejj@ximian.com>
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -637,6 +671,7 @@ base64_init(void)
 	}
 	camel_mime_base64_rank['='] = 0;
 }
+#endif
 
 /**
  * gsf_base64_encode_close :
@@ -650,14 +685,20 @@ base64_init(void)
  * This funcion should be called to when finished encoding everything, to
  * flush off the last little bit.
  *
- * Returns :
- */
+ * Returns: a count of the number of bytes in the final block.
+ **/
 size_t
 gsf_base64_encode_close (guint8 const *in, size_t inlen,
 			 gboolean break_lines, guint8 *out, int *state, unsigned int *save)
 {
-	int c1, c2;
 	guint8 *outptr = out;
+#ifdef HAVE_G_BASE64_ENCODE
+	if (inlen > 0)
+		outptr += gsf_base64_encode_step (in, inlen, break_lines,
+						  outptr, state, save);
+	outptr += g_base64_encode_close (break_lines, outptr, state, save);
+#else
+	int c1, c2;
 
 	if (inlen>0)
 		outptr += gsf_base64_encode_step(in, inlen, break_lines, outptr, state, save);
@@ -691,6 +732,7 @@ gsf_base64_encode_close (guint8 const *in, size_t inlen,
 	*save = 0;
 	*state = 0;
 
+#endif
 	return outptr-out;
 }
 
@@ -707,12 +749,15 @@ gsf_base64_encode_close (guint8 const *in, size_t inlen,
  * the output @out at a time, saves left-over state in @state and @save
  * (initialise to 0 on first invocation).
  *
- * Returns : the number of bytes encoded
+ * Returns: the number of bytes encoded
  */
 size_t
 gsf_base64_encode_step (guint8 const *in, size_t len,
 			gboolean break_lines, guint8 *out, int *state, unsigned int *save)
 {
+#ifdef HAVE_G_BASE64_ENCODE
+	return g_base64_encode_step (in, len, break_lines, out, state, save);
+#else
 	register guint8 const *inptr;
 	register guint8 *outptr;
 
@@ -784,6 +829,7 @@ gsf_base64_encode_step (guint8 const *in, size_t len,
 		 (int)((char *)save)[2]));
 
 	return outptr-out;
+#endif
 }
 
 
@@ -797,12 +843,15 @@ gsf_base64_encode_step (guint8 const *in, size_t len,
  *
  * Decodes a chunk of base64 encoded data
  *
- * Returns the number of bytes converted
+ * Returns: the number of bytes converted
  **/
 size_t
 gsf_base64_decode_step (guint8 const *in, size_t len, guint8 *out,
 			int *state, guint *save)
 {
+#ifdef HAVE_G_BASE64_ENCODE
+	return g_base64_decode_step (in, len, out, state, save);
+#else
 	register guint8 const *inptr;
 	register guint8 *outptr, c;
 	register unsigned int v;
@@ -847,6 +896,7 @@ gsf_base64_decode_step (guint8 const *in, size_t len, guint8 *out,
 
 	/* if i!= 0 then there is a truncation error! */
 	return outptr-out;
+#endif
 }
 
 /**
@@ -856,23 +906,23 @@ gsf_base64_decode_step (guint8 const *in, size_t len, guint8 *out,
  *
  * Encodes data from @data back into @data using base64 encoding.
  *
- * Returns : the number of bytes encoded
+ * Returns: the number of bytes encoded
  */
 guint8 *
 gsf_base64_encode_simple (guint8 const *data, size_t len)
 {
 	guint8 *out;
-	int state = 0, outlen;
-	unsigned int save = 0;
-	gboolean break_lines = TRUE;
+	int state = 0;
+	guint save = 0;
+	gboolean break_lines = TRUE;  /* This differs from g_base64_encode */
+	size_t outlen = len * 4 / 3 + 5;
 
-	outlen = len * 4 / 3 + 5;
 	if (break_lines)
-		outlen += outlen / BASE64_LINE_LEN + 1;
+		outlen += outlen / 72 + 1;
 	out = g_new (guint8, outlen);
 	outlen = gsf_base64_encode_close (data, len, break_lines,
 					  out, &state, &save);
-	out [outlen] = '\0';
+	out[outlen] = '\0';
 	return out;
 }
 
@@ -883,13 +933,13 @@ gsf_base64_encode_simple (guint8 const *data, size_t len)
  *
  * Decodes a chunk of base64 encoded data from @data back into @data.
  *
- * Returns : the number of bytes converted
+ * Returns: the number of bytes converted
  */
 size_t
 gsf_base64_decode_simple (guint8 *data, size_t len)
 {
 	int state = 0;
-	unsigned int save = 0;
+	guint save = 0;
 	return gsf_base64_decode_step (data, len, data, &state, &save);
 }
 
@@ -901,11 +951,11 @@ gsf_base64_decode_simple (guint8 *data, size_t len)
  * @p_n_params: a pointer to the number of properties collected.  (Used for
  *   both input and output.)
  * @p_params: a pointer to the GParameter array that holds the properties.
- *   (Used for both input and output.  This may point to a NULL pointer if
+ *   (Used for both input and output.  This may point to a %NULL pointer if
  *   there are no properties collected yet.)
  * @first_property_name: the name of the first property being set, or NULL.
  * @var_args: a va_list holding the remainder of the property names and
- *   values, terminated by a NULL.
+ *   values, terminated by a %NULL.
  *
  * This function builds a GParameter array suitable for g_object_newv.
  **/
@@ -996,10 +1046,8 @@ gsf_property_settings_free (GParameter *params,
 /**
  * gsf_error_quark:
  *
- * Returns the #GQuark used to identify libgsf errors in #GError structures.
- * Specific error codes come from the #GsfError enumeration.
- * 
- * Return value: A #GQuark.
+ * Returns:  the #GQuark used to identify libgsf errors in #GError structures.
+ * 	Specific error codes come from the #GsfError enumeration.
  **/
 GQuark
 gsf_error_quark (void)
